package com.bankaudit.dao;

import java.util.List;

import com.bankaudit.dto.DataTableResponse;
import com.bankaudit.model.MaintActionItemWrk;

public interface MaintActionItemsDao extends Dao {
	
	
	DataTableResponse getAllActionItemList(Integer legalEntityCode, String search,Integer orderColumn, String orderDirection, Integer page, Integer size);

	DataTableResponse getActionItemListByauditTypeCode(Integer legalEntityCode, String auditTypeCode, String status, String search, Integer orderColumn, String orderDirection, 
			Integer page, Integer size);
	
	List<MaintActionItemWrk> getAllActionItemListByauditTypeCode(Integer legalEntityCode, String auditTypeCode, String status);
	 	 
	void copyDataFromMasterToWrk(Integer legalEntityCode, String auditTypeCode, String userId);
	
	void updateMaker(Integer legalEntityCode, String auditTypeCode, String userId, String status);
	 	
	void deleteFromMaintActionItem(Integer legalEntityCode, String auditTypeCode, Integer id);
	
	void deleteFromMaintActionItemWrk(Integer legalEntityCode, String auditTypeCode, Integer id);
	
	MaintActionItemWrk getModifiedDataFromWrk(Integer legalEntityCode, String auditTypeCode, Integer masterId);

	// void deleteActionItemById(Integer legalEntityCode, String auditTypeCode, Integer id, String status);

}
